/*****************************************************************************/
/*****************************************************************************/
/**                                                                         **/
/**      IBM TotalStorage Tape & Medium Changer Device Driver on HP-UX      **/
/**           (C) COPYRIGHT IBM Corporation, 1998, 2011                     **/
/**                                                                         **/
/**            st.h - IBM SCSI Tape Drive Interface Include File            **/
/**                    Date: 06/15/2011  version:114                        **/
/**                  Licensed Materials - Property of IBM                   **/
/**                           All Rights Reserved                           **/
/**                                                                         **/
/**    US Government Users Restricted Rights  -  Use, duplication, or       **/
/**    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.    **/
/**                                                                         **/
/*****************************************************************************/
/*****************************************************************************/

#ifndef ATDD_ST_H
#define ATDD_ST_H

#include <sys/mtio.h>

#ifndef UCHAR
#define UCHAR
typedef unsigned char   uchar;
#endif

#ifndef ULLONG
#define ULLONG
typedef unsigned long long ullong;
#endif

/*===========================================================================*/
/* Basic SCSI IOCTL Command Definitions and Associated Structures            */
/*===========================================================================*/

/*---------------------------------------------------------------------------*/
/* Defines and data structures for INQUIRY                                   */
/*---------------------------------------------------------------------------*/

#define IOC_INQUIRY                     _IOR('S',1,at_inquiry_data_t)

typedef struct {
  uchar qual            : 3,            /* peripheral qualifier              */
        type            : 5;            /* device type                       */
  uchar rm              : 1,            /* removable medium                  */
        mod             : 7;            /* device type modifier              */
  uchar iso             : 2,            /* ISO version                       */
        ecma            : 3,            /* EMCA version                      */
        ansi            : 3;            /* ANSI version                      */
  uchar aen             : 1,            /* asynchronous even notification    */
        trmiop          : 1,            /* terminate I/O process message     */
                        : 2,            /* reserved                          */
        rdf             : 4;            /* response data format              */
  uchar len;                            /* additional length                 */
  uchar                 : 8;            /* reserved                          */
  uchar                 : 1,            /* reserved                          */
        encsrv          : 1,            /* enclosure service                 */
        barcod          : 1,            /* bar code scanner attached         */
        multip          : 1,            /* multi-port                        */
        mchngr          : 1,            /* medium changer mode               */
                        : 3;            /* reserved                          */
  uchar reladr          : 1,            /* relative addressing               */
        wbus32          : 1,            /* 32 bit wide data transfers        */
        wbus16          : 1,            /* 16 bit wide data transfers        */
        sync            : 1,            /* synchronous data transfers        */
        linked          : 1,            /* linked commands                   */
                        : 1,            /* reserved                          */
        cmdque          : 1,            /* command queueing                  */
        sftre           : 1;            /* soft reset                        */
  uchar vid[8];                         /* vendor ID                         */
  uchar pid[16];                        /* product ID                        */
  uchar rev[4];                         /* product revision level            */
  uchar vendor[92];                     /* vendor specific (padded to 128)   */
} at_inquiry_data_t;

/*---------------------------------------------------------------------------*/
/* Defines and data structures for REQUEST_SENSE                             */
/*---------------------------------------------------------------------------*/

#define IOC_REQUEST_SENSE               _IOR('T',2,at_sense_data_t)

typedef struct {
  uchar valid           : 1,            /* sense data is valid               */
        code            : 7;            /* error code                        */
  uchar segnum;                         /* segment number                    */
  uchar fm              : 1,            /* file mark detected                */
        eom             : 1,            /* end of medium                     */
        ili             : 1,            /* incorrect length indicator        */
                        : 1,            /* reserved                          */
        key             : 4;            /* sense key                         */
  uchar info[4];                        /* information bytes                 */
  uchar addlen;                         /* additional sense length           */
  uchar cmdinfo[4];                     /* command specific information      */
  uchar asc;                            /* additional sense code             */
  uchar ascq;                           /* additional sense code qualifier   */
  uchar fru;                            /* field replaceable unit code       */
  uchar sksv            : 1,            /* sense key specific valid          */
        cd              : 1,            /* control/data                      */
                        : 2,            /* reserved                          */
        bpv             : 1,            /* bit pointer valid                 */
        sim             : 3;            /* system information message        */
  uchar field[2];                       /* field pointer                     */
  uchar vendor[110];                    /* vendor specific (padded to 128)   */
} at_sense_data_t;


/*---------------------------------------------------------------------------*/
/* Defines for TEST_UNIT_READY                                               */
/*---------------------------------------------------------------------------*/

#define IOC_TEST_UNIT_READY             _IO('S',3)


/*---------------------------------------------------------------------------*/
/* Defines for RESERVE and RELEASE                                           */
/*---------------------------------------------------------------------------*/

#define IOC_RESERVE                     _IO('S',4)

#define IOC_RELEASE                     _IO('S',5)


/*---------------------------------------------------------------------------*/
/* Defines and data structures for LOG_SENSE_PAGE and LOG_SENSE10_PAGE       */
/*---------------------------------------------------------------------------*/

#define IOC_LOG_SENSE_PAGE              _IOWR('S',6, log_sns_pg_t)

#define MAX_LGPGDATA 250                /* The maximum data length which     */
                                        /* this ioctl can return, including  */
                                        /* the log page header.              */

typedef struct {
   uchar  page_code;                    /* page code                         */
   uchar  data[MAX_LGPGDATA];           /* log data structure                */
} log_sns_pg_t;

#define LOGSENSEPAGE 1024
#define IOC_LOG_SENSE10_PAGE      	 _IOWR('S',12, log_sense10_page_t)

typedef struct {
   uchar page_code;			    	/* [IN] Log sense page */
   uchar subpage_code;		        /* [IN] Log sense subpage */
   uchar reserved[2];			    /* unused */
   unsigned short len;			    /* [OUT] number of valid bytes in data (log_page_header_size + page_length) */
   unsigned short parm_pointer;		/* [IN] specific parameter number at which the data begins */
   char data[LOGSENSEPAGE];	    	/* [OUT] log data */
} log_sense10_page_t;


/*
** Log Page Data Structures
**
** Portions of the log_sns_pg_t data[] field may be
** cast to these structures when interpreting the mode data which
** is returned by IOC_LOG_SENSE_PAGE.
**
** Here is the generalized structure of a log page:
**
** Log Page
**    Log Page Header
**       Page Code
**       Page Length
**    Log Parameter(s) (One or more may exist)
**       Parameter Code
**       Control Byte
**       Parameter Length
**       Parameter Value
*/

typedef struct {
   uchar page_code;                     /* page code                         */
   uchar reserved_1;                    /* reserved                          */
   uchar len[2];                        /* page length                       */
} log_page_hdr_t;

typedef struct {
   uchar parm_code[2];                  /* parameter code                    */
   uchar ctl_byte;                      /* control byte                      */
   uchar parm_len;                      /* parameter length                  */
} log_parm_hdr_t;


/*---------------------------------------------------------------------------*/


typedef struct {
   log_page_hdr_t page_hdr;
   log_parm_hdr_t parm_hdr;
   uchar alert_value;                  /* tape alert flag                    */
} log_pg_2E_t;

/*SIM/MIM page*/
typedef struct {
   log_page_hdr_t page_hdr;
   log_parm_hdr_t parm_hdr;
   uchar indicator;                    /* SIM MIM indicator                  */
   uchar res1[11];                     /* reserved                           */
   uchar mcode[2];                     /* message code                       */
   uchar res2[2];                      /* reserved                           */
   uchar exception_mcode;              /* exception message code             */
   uchar service_mcode;                /* service message code               */
   uchar severity_code;                /* message severity code              */
   uchar res3;                         /* reserved                           */
   uchar exception_data[2];            /* exception data                     */
   uchar fru_id[3];                    /* FRU id in SIM or FSC in MIM        */
   uchar fsc1[4];                      /* first FSC                          */
   uchar fsc2[4];                      /* last FSC                           */
   uchar product_id[4];                /* product ID                         */
   uchar product_ident[18];            /* product identifier                 */
   uchar device_type[8];               /* device type and model number       */
} log_pg_31_t;


/*---------------------------------------------------------------------------*/
/* Defines and data structures for MODE_SENSE and MODE_PAGE                  */
/*---------------------------------------------------------------------------*/

#define IOC_MODE_SENSE                 _IOWR('S',7, mode_sns_t)

#define IOC_MODE_SENSE_WITH_SUBPAGE	   _IOWR('S',13, mode_sns_sub_t)

#define MAX_MSDATA 253                 /* The maximum data length which this */
                                       /* ioctl can return, including        */
                                       /* headers and block descriptors.     */

#define ATDD_MODE_SENSE_PAGE  255

#define MODESNS_10_CMD 0x5A            /* SCSI cmd code for 10-byte version  */
                                       /* of the command                     */

#define MODESNS_6_CMD  0x1A            /* SCSI cmd code for 6-byte version   */
                                       /* of the command                     */

typedef struct {
   uchar page_code;                    /* Page Code:  Set this field with    */
                                       /*    the desired mode page number    */
                                       /*    before issuing the ioctl.       */
   uchar cmd_code;                     /* SCSI Command Code:  Upon return,   */
                                       /*    this field is set with the      */
                                       /*    SCSI command code to which      */
                                       /*    the device responded.           */
                                       /*    x'5A' = Mode Sense (10)         */
                                       /*    x'1A' = Mode Sense (6)          */
   uchar data[MAX_MSDATA];             /* Mode Parameter List:  Upon return, */
                                       /*    this field contains the mode    */
                                       /*    parameters list, up to the max  */
                                       /*    length supported by the ioctl.  */
} mode_sns_t;

typedef struct {
   uchar page_code;                    /* Page Code:  Set this field with    */
                                       /*    the desired mode page number    */
                                       /*    before issuing the ioctl.       */
   uchar subpage_code;	               /* Subpage Code: Set this field with  */
   			                           /*    the desired mode subpage number */
                                       /*    before issuing the ioctl.       */
   uchar reserved[6];
   uchar cmd_code;                     /* SCSI Command Code:  Upon return,   */
                                       /*    this field is set with the      */
                                       /*    SCSI command code to which      */
                                       /*    the device responded.           */
                                       /*    x'5A' = Mode Sense (10)         */
                                       /*    x'1A' = Mode Sense (6)          */
   uchar data[ATDD_MODE_SENSE_PAGE];   /* Mode Parameter List:  Upon return, */
                                       /*    this field contains the mode    */
                                       /*    parameters list, up to the max  */
                                       /*    length supported by the ioctl.  */
} mode_sns_sub_t;

/*
** Mode Parameter List Data Structures
**
** Portions of the mode_sns_t data[] field may be cast to
** these structures when interpreting the mode parameter list
** returned by IOC_MODE_SENSE.
**
** Here is the generalized structure of a mode parmameter list:
**
** Mode Parameter List
**    Mode Parameter Header (One of the two possible versions)
**       Data Length
**       Medium Type
**       Device-Specific Parameter
**       Block Descriptor Length
**    Block Descriptor(s) (Zero or more may be present)
**       Density Code
**       Block Count
**       Block Length
**    Mode Page(s) (One or more may be present)
**       Page Code
**       Page Length
**       Mode Parameters
**
** The Mode Parameter Header begins at byte 0 of the
** mode_sns_t data[] field.
*/

/* Mode Parameter Header for 10-byte version of Mode Sense.                  */
typedef struct {
   uchar data_len[2];                  /* Mode Data Length which follows     */
                                       /* this field                         */
   uchar medium_type;                  /* Medium Type                        */
   uchar wrt_prot        : 1,          /* Write Protect                      */
         buf_mode        : 3,          /* Buffered Mode                      */
         speed           : 4;          /* Speed                              */
   uchar reserved1[2];                 /* Reserved                           */
   uchar blk_dsc_len[2];               /* Block Descriptor Length            */
} mode_hdr10_t;

/* Mode Parameter Header for 6-byte version of Mode Sense.                   */
typedef struct {
   uchar data_len;                     /* Mode Data Length which follows     */
                                       /*    this field                      */
   uchar medium_type;                  /* Medium Type                        */
   uchar wrt_prot        : 1,          /* Write Protect                      */
         buf_mode        : 3,          /* Buffered Mode                      */
         speed           : 4;          /* Speed                              */
   uchar blk_dsc_len;                  /* Block Descriptor Length            */
} mode_hdr6_t;

/* If blk_dsc_len is non-zero, then one or more block descriptors exists.    */
/* Number of block descriptors = blk_desc_len / sizeof(blkdsc_t)             */
typedef struct {
   uchar density_code;                 /* Density Code                       */
   uchar blk_count[3];                 /* Number of Blocks                   */
   uchar reserved1;                    /* Reserved                           */
   uchar blk_len[3];                   /* Block Length                       */
} blkdsc_t ;

/*
** The maximum length of mode page data which can be returned by
** IOC_MODE_SENSE depends on whether the six-byte or ten-byte version
** of Mode Sense is used by the tape device, and whether or not the
** returned mode parameter list includes one or more block descriptors.
**
** For any given call to IOC_MODE_SENSE, the maximum length of
** returned mode data can be calculated as:
**
**    MAX_MSDATA
**       - <size_of_mode_parameter_header_structure>
**       - <block_descriptor_length_in_mode_parameter_header>
**
** This maximum holds, regardless of the Page Length value indicated in the
** mode page itself.
**
** The first two bytes of the mode page data may be interpreted using the
** following structure.  All remaining bytes must be interpreted according
** to the description of each individual mode page in the appropriate
** hardware reference.
*/
typedef struct {
   uchar ps            : 1,            /* Parameter Saveable                 */
         reserved_1    : 1,            /* reserved                           */
         page_code     : 6;            /* Page Code                          */
   uchar page_len;                     /* Page Length of following data      */
} modep_prefix_t;


/*---------------------------------------------------------------------------*/
/* Defines and data structures for INQUIRY_PAGE                              */
/*---------------------------------------------------------------------------*/

#define IOC_INQUIRY_PAGE               _IOWR('S',8, inquiry_page_t)

#define MAX_IPDATA   253               /* max data xfer for inquiry pg ioctl */

typedef struct {
  uchar page_code;                     /* page code                          */
  uchar data[MAX_IPDATA];              /* inquiry parameter List             */
} inquiry_page_t;

typedef struct {
   uchar periph_qual : 3,              /* peripheral qualifier               */
         periph_type : 5;              /* peripheral device type             */
   uchar page_code;                    /* page code                          */
   uchar reserved_1;                   /* reserved                           */
   uchar page_len;                     /* page length                        */
   uchar serial[12];                   /* serial number                      */
} inq_pg_80_t;

typedef struct {
   uchar periph_qual : 3,              /* peripheral qualifier               */
         periph_type : 5;              /* peripheral device type             */
   uchar page_code;                    /* page code                          */
   uchar reserved_1;                   /* reserved                           */
   uchar page_len;                     /* page length                        */
   uchar vendor[2];                    /* vendor-specific world wide name    */
   uchar port;                         /* port node ID                       */
} inq_pg_83_t;


/*---------------------------------------------------------------------------*/
/* Defines for PREVENT and ALLOW_MEDIUM_REMOVAL                              */
/*---------------------------------------------------------------------------*/

#define IOC_PREVENT_MEDIUM_REMOVAL      _IO('S', 9)

#define IOC_ALLOW_MEDIUM_REMOVAL        _IO('S', 10)


/*---------------------------------------------------------------------------*/
/* Defines and data structures for GET_DRIVER_INO                            */
/*---------------------------------------------------------------------------*/

#define IOC_GET_DRIVER_INFO             _IOR('S',11,get_driver_info_t)

typedef struct {
    char driver_id[64];               /* the name of the tape driver (ATDD) */
    char version[25];                 /* the version of the tape driver     */
} get_driver_info_t;

/*----------------------------------------------------------------------------*/


/*============================================================================*/
/* SCSI Tape Drive IOCTL Command Definitions and Associated Structures        */
/*============================================================================*/

/*---------------------------------------------------------------------------*/
/* Defines and data structures for STIOC_TAPE_OP                             */
/*                                                                           */
/*                 ---------- NOTE -----------                               */
/* To compile the code on HP-UX 10.20, mtio.h in PHKL_22286 or later is      */
/* requested.                                                                */
/* MT and ST arguments separate to issue the MTIOCTOP and STIOC_TAPE_OP      */
/* ioctls.                                                                   */
/*---------------------------------------------------------------------------*/

#define STIOC_TAPE_OP                  _IOWR('T',1,tape_op_t)
/* identical to MTIOCTOP in mtio.h     */

/* Structure for STIOC_TAPE_OP ioctl - mag tape op command */
typedef struct {
     short st_op;                      /* st operations defined below */
     daddr_t st_count;                 /* how many of them  */
} tape_op_t;

#define ST_OP_WEOF        10000        /* write filemark (end-of-file record) */
#define ST_OP_FSF         10001        /* forward space file */
#define ST_OP_BSF         10002        /* backward space file */
#define ST_OP_FSR         10003        /* forward space record */
#define ST_OP_BSR         10004        /* backward space record */
#define ST_OP_REW         10005        /* Rewind  */
#define ST_OP_OFFL        10006        /* Offline  */
#define ST_OP_NOP         10007        /* no operation, may set status */
#define ST_OP_RETEN       10008        /* Retension    */
#define ST_OP_EOD         10010        /* End of Data */
#define ST_OP_NBSF        10011        /* position the tape to the beginning of previous file */
#define ST_OP_SRSZ        10012        /* set record size   */
#define ST_OP_GRSZ        10013        /* get record size  */
#define ST_OP_RES         10014        /* Resever */
#define ST_OP_REL         10015        /* Release  */
#define ST_OP_ERASE       10016        /* Erase  */
#define ST_OP_LOAD        10020        /* load tape in drive  */
#define ST_OP_UNLOAD      10021        /* unload tape from drive */


/*---------------------------------------------------------------------------*/
/* Defines and data structures for GET_DEVICE_STATUS                         */
/*---------------------------------------------------------------------------*/

#define STIOC_GET_DEVICE_STATUS         _IOR('T',2,device_status_t)
                                        /* identical to MTIOCGET in mtio.h   */

typedef struct mtget device_status_t;

#define STF_RACL        0x10            /* random access cartridge loader    */
#define STF_ACL         0x20            /* automatic cartridge loader        */


/*---------------------------------------------------------------------------*/
/* Defines and data structures for GET_DEVICE_INFO                           */
/*---------------------------------------------------------------------------*/

#define STIOC_GET_DEVICE_INFO           _IOR('T',3,device_info_t)

typedef struct {
   char  name[64];                    /* Drive Name                       */
   char  vid[25];                     /* Vendor id and model (product) id */
   char  type;                        /* Drive type for driver */
   int   bsize;                       /* Block size */
   int   options;                     /* Drive options */
   int   max_rretries;                /* Max read retries */
   int   max_wretries;                /* Max write retries */
   uchar default_density;             /* Default density chosen */
} device_info_t;

/*---------------------------------------------------------------------------*/
/* Defines and data structures for GET_MEDIA_INFO                            */
/*---------------------------------------------------------------------------*/

#define STIOC_GET_MEDIA_INFO            _IOR('T',4,media_info_t)

typedef struct {
   uint  media_type;                    /* type of media loaded              */
   uint  media_format;                  /* format of media loaded            */
   uchar write_protect;                 /* write protect (physical/logical)  */
} media_info_t;


/* media type from Mode Parameter Header */
#define SLT_CST_CART     0x00000110      /* IBM short length cartridge               */
#define MLT_CST_CART     0x00000150      /* IBM marginal length cartridge            */
#define EC_CST_CART      0x00000190      /* IBM 3480/3490E/3490C cartridge           */
#define IBM_3570B_CART   0x00000042      /* IBM Magstar MP FAL Tape B Cartridge      */
#define IBM_3570C_CART   0x00000043      /* IBM Magstar MP FAL Tape C Cartridge      */
#define IBM_3570CXL_CART 0x00000044      /* IBM Magstar MP FAL Tape C XL Cartridge   */

#define HIGH_PER_CART    0x00000081      /* IBM 3590 High Performance Cart           */
#define EXTH_PER_CART    0x00000082      /* IBM 3590 Extended High Performance Cart  */
#define IBM_3592_CART    0x00000090      /* IBM 3592 Enterprise Tape Cartridge       */
#define IBM_3592JA_CART  0x00000091      /* IBM 3592 Enterprise Tape Cartridge       */
#define IBM_3592JB_CART  0x00000092      /* IBM 3592 GenB Cartridge                  */
#define IBM_3592JW_CART  0x000000A1      /* IBM 3592 Enterprise Tape Worm Cartridge  */
#define IBM_3592JX_CART  0x000000A2      /* IBM 3592 GenB WORM Cartridge             */
#define IBM_3592JJ_CART  0x000000B1      /* IBM 3592 Enterprise Tape Short Cartridge */
#define IBM_3592JK_CART  0x000000B2      /* IBM 3592 GenB Short Cartridge            */
#define IBM_3592JR_CART  0x000000C1      /* IBM 3592 Enterprise Tape Short Worm Cart */
#define IBM_3592JS_CART  0x000000C2      /* IBM 3592 GenB Short WORM Cartridge       */
#define IBM_3592JC_CART  0x000000D1      /* IBM 3592 E07  R/W Data Media Cartridge   */
//#define IBM_3592JK_CART  0x000000D2      /* IBM 3592 E07 Short WORM Media Cartridge  */
#define IBM_3592JY_CART  0x000000D3      /* IBM 3592 E07 WORM Media Cartridge        */

/* Defines for media_type from Report Density Support */
#define IBM_LTO1_CART      0x00000018    /* IBM Ultrium LTO-1 U-18 Tape Cartridge    */
#define IBM_LTO2_CART      0x00000028    /* IBM Ultrium LTO-2 U-28 Tape Cartridge    */
#define IBM_LTO3_CART      0x00000038    /* IBM Ultrium LTO-3 U-316 Tape Cartridge   */
#define IBM_LTO3_WORM_CART 0x0000003C    /* IBM Ultrium LTO-3 U-316 LT WORM Cartridge*/
#define IBM_LTO4_CART      0x00000048    /* IBM Ultrium LTO-4 Tape Cartridge         */
#define IBM_LTO4_WORM_CART 0x0000004C    /* IBM Ultrium LTO-4 Tape LU WORM Cartridge */
#define IBM_LTO5_CART      0x00000058    /* IBM Ultrium LTO-5 Tape Cartridge         */
#define IBM_LTO5_WORM_CART 0x0000005C    /* IBM Ultrium LTO-5 Tape WORM Cartridge    */
#define IBM_LTO6_CART      0x00000068    /* IBM Ultrium LTO-6 Tape Cartridge         */
#define IBM_LTO6_WORM_CART 0x0000006C    /* IBM Ultrium LTO-6 Tape WORM Cartridge    */


/* media format */
#define BASIC_FMT        0x02000000      /* IBM 3480/3490 basic format        */
#define COMPACT_FMT      0x04000000      /* IBM 3480/3490 compacted format    */
/* The following defines from Mode Block Descriptor */
#define IBM_3570B_FMT    0x00000082      /* IBM Magstar MP FAL Tape B Format  */
#define IBM_3570C_FMT    0x00000083      /* IBM Magstar MP FAL Tape C Format  */
#define IBM_3590B_FMT    0x00000029      /* IBM 3590 B-Series model (B-Format)*/
#define IBM_3590E_FMT    0x0000002A      /* IBM 3590 E-series model (E-Format)*/
#define IBM_3590H_FMT    0x0000002C      /* IBM 3590 H-series model (H-Format)*/
#define IBM_3592_FMT     0x00000051      /* IBM 3592 J1A Model Format         */
#define IBM_3592E_FMT    0x00000052      /* IBM 3592 E05 Model Format         */
#define IBM_3592J_EFMT   0x00000071      /* IBM 3592 J1A Encryption Format    */
#define IBM_3592E_EFMT   0x00000072      /* IBM 3592 E05 Encryption Format    */
#define IBM_3592E06_EFMT 0x00000073      /* IBM 3592 E06 Encryption Format    */
#define IBM_3592E07_EFMT 0x00000074      /* IBM 3592 E07 Encryption Format    */
#define LTO_18T_FMT      0x00000040      /* Ultrium_1/8T Format               */
#define LTO_28T_FMT      0x00000042      /* Ultrium_2/8T Format               */
#define LTO_316T_FMT     0x00000044      /* Ultrium_3/16T Format              */
#define LTO_416T_FMT     0x00000046      /* Ultrium_4/16T Format              */
#define LTO_516T_FMT     0x00000058      /* Ultrium_5/16T Format              */
#define LTO_616T_FMT     0x0000005A      /* Ultrium_6/16T Format                     */


/*---------------------------------------------------------------------------*/
/* Defines and data structures for GET_POSITION and SET_POSITION             */
/*---------------------------------------------------------------------------*/

#define STIOC_GET_POSITION              _IOWR('T',5,position_data_t)

#define STIOC_SET_POSITION              _IOW('T',6,position_data_t)

typedef struct {
   uchar block_type;                     /* block type (logical or physical)  */
   uchar bot;                            /* physical beginning of tape        */
   uchar eot;                            /* logical end of tape               */
   uchar partition;                      /* partition number                  */
   uint  position;                       /* current or new block ID           */
   uint  last_block;                     /* last block written to tape        */
   uint  block_count;                    /* blocks remaining in buffer        */
   uint  byte_count;                     /* bytes remaining in buffer         */
} position_data_t;

#define LOGICAL_BLK     1               /* SCSI logical block ID format      */
#define PHYSICAL_BLK    2               /* vendor specific block ID format   */


/*---------------------------------------------------------------------------*/
/* Defines and data structures for GET_PARM and SET_PARM                     */
/*---------------------------------------------------------------------------*/

#define STIOC_GET_PARM                  _IOWR('T',7,parm_data_t)

#define STIOC_SET_PARM                  _IOW('T',8,parm_data_t)

typedef struct {
   uchar type;                          /* type of parameter to get or set   */
   uint  value;                         /* current or new value of parameter */
} parm_data_t;

#define BLOCKSIZE         1             /* block size (0 -> 2097152 [2 meg]) */
#define COMPRESSION       2             /* compression mode (ON/OFF)         */
#define BUFFERING         3             /* buffering mode (ON/OFF)           */
#define IMMEDIATE         4             /* immediate mode (ON/OFF)           */
#define TRAILER           5             /* trailer label mode (ON/OFF)       */
#define WRITEPROTECT      6             /* phys/log write protect mode       */
#define ACFMODE           7             /* automatic cartridge facility mode */
#define SCALING           8             /* capacity scaling mode             */
#define SILI              9             /* illegal length ind mode (ON/OFF)  */
#define RELEASE_ON_CLOSE 10             /* release on close                  */
#define DATASAFE         11             /* data safe mode                    */
#define PEWSIZE          12             /* Programmable early warning size   */

#define OFF               0             /* disable mode                      */
#define ON                1             /* enable mode                       */

#define NO_PROTECT      0x00            /* no write protect                  */
#define WORM_PROTECT    0x01            /* worm write protect                */
#define PERS_PROTECT    0x10            /* persistant write protect          */
#define ASSC_PROTECT    0x40            /* associated write protect          */
#define PHYS_PROTECT    0x80            /* physical write protect            */

#define NO_ACF          0               /* no ACF present                    */
#define SYSTEM_MODE     1               /* system ACF mode                   */
#define RANDOM_MODE     2               /* random ACF mode                   */
#define MANUAL_MODE     3               /* manual ACF mode                   */
#define ACCUM_MODE      4               /* accumulate ACF mode               */
#define AUTO_MODE       5               /* automatic ACF mode                */
#define LIB_MODE        6               /* library ACF mode                  */

#define SCALE_100       0xF0            /* 100% capacity                     */
#define SCALE_75        0xC0            /* 75% capacity                      */
#define SCALE_50        0x80            /* 50% capacity                      */
#define SCALE_25        0x40            /* 25% capacity                      */


/*---------------------------------------------------------------------------*/
/* Defines and data structures for DISPLAY_MSG                               */
/*---------------------------------------------------------------------------*/

#define STIOC_DISPLAY_MSG               _IOW('T',9,msg_data_t)

#define MSGLEN          8               /* max message length                */

typedef struct {
   uchar function;                      /* message function code             */
   char  msg_0[MSGLEN];                 /* message 0                         */
   char  msg_1[MSGLEN];                 /* message 1                         */
} msg_data_t;

#define DISPMSG0        0x00            /* display message 0                 */
#define DISPMSG1        0x04            /* display message 1                 */
#define FLASHMSG0       0x08            /* flash message 0                   */
#define FLASHMSG1       0x0C            /* flash message 1                   */
#define ALTERNATE       0x10            /* alternate message 0 & message 1   */

#define GENSTATUS       0x00            /* general status message            */
#define DMNTVERIFY      0x20            /* demount-verify message            */
#define MNTIMMED        0x40            /* mount immediate indicator         */
#define DMNTIMMED       0xE0            /* demount immediate indicator       */


/*---------------------------------------------------------------------------*/
/* Defines for SYNC_BUFFER                                                   */
/*---------------------------------------------------------------------------*/

#define STIOC_SYNC_BUFFER               _IO('T',10)


/*---------------------------------------------------------------------------*/
/* Defines and data structures for REPORT_DENSITY_SUPPORT                    */
/*---------------------------------------------------------------------------*/

#define STIOC_REPORT_DENSITY_SUPPORT  _IOWR('T', 11, rpt_dens_sup_t)


#define ALL_MEDIA_DENSITY      0
#define CURRENT_MEDIA_DENSITY  1

#define MAX_DENSITY_REPORTS    4

typedef struct {
   uchar  pri_density_code;       /* primary density code                   */
   uchar  sec_density_code;       /* secondary densuty code                 */
   uchar  wrtok     : 1,          /* write ok, device can write this format */
          dup       : 1,          /* zero if density only reported once     */
          deflt     : 1,          /* current density is default format      */
          res_1     : 5;          /* reserved                               */
   uchar  reserved1[2];           /* reserved                               */
   uchar  bits_per_mm[3];         /* bits per mm                            */
   uchar  media_width[2];         /* media width in millimeters             */
   uchar  tracks[2];              /* tracks                                 */
   uchar  capacity[4];            /* capacity in megabytes                  */
   char   assigning_org[8];       /* assigning organization in ASCII        */
   char   density_name[8];        /* density name in ASCII                  */
   char   description[20];        /* description in ASCII                   */
} density_report_t;

typedef struct {
   uchar  media;             /* report all or current media as defined above */
   uchar  count;             /* number of density reports returned in array  */
   density_report_t reports[MAX_DENSITY_REPORTS];
} rpt_dens_sup_t;


/*---------------------------------------------------------------------------*/
/* Defines for PREVENT and ALLOW_MEDIUM_REMOVAL                              */
/*---------------------------------------------------------------------------*/

#define STIOC_PREVENT_MEDIUM_REMOVAL      IOC_PREVENT_MEDIUM_REMOVAL

#define STIOC_ALLOW_MEDIUM_REMOVAL        IOC_ALLOW_MEDIUM_REMOVAL


/*----------------------------------------------------------------------------*/
/* Defines and data structures for PERS_RESERVE_IN (Persistent Reserve In)    */
/*----------------------------------------------------------------------------*/

#define STIOC_PERS_RESERVE_IN           _IOWR('T',12, pers_reserve_in_t)

#define ALL_REGISTERED_RESERVATION_KEYS       0
#define ALL_CURRENT_PERSISTENT_RESERVATIONS   1

typedef struct {
   uchar service_action;               /* service action                      */
   uchar len[3];                       /* allocation length                   */
   union {                             /* persist reserve in parameter data   */
     struct {
       uchar generation[4];           /* a counter for PERSISTENT RESERVE OUT cmd requests */
       uchar additional_length[4];    /* a count of the number bytes in Reservation key list */
       #ifdef __LP64__
          ptr32_t reserve_key_list;   /* pointer to reservation key list     */
       #else
          void *reserve_key_list;     /* pointer to reservation key list     */
       #endif /* __LP64__ */
     } read_key;

     struct {
       uchar generation[4];           /* a counter for PERSISTENT RESERVE OUT cmd requests */
       uchar additional_length[4];    /* a count of the number bytes in Reservation key list */
       #ifdef __LP64__
          ptr32_t reserve_descriptors;  /* pointer to all of reservation descriptors  */
       #else
          void *reserve_descriptors;    /* pointer to all of reservation descriptors  */
       #endif /* __LP64__ */
     } read_reservations;
   } parm_data;
} pers_reserve_in_t;


/*----------------------------------------------------------------------------*/
/* Defines and data structures for PERS_RESERVE_OUT (Persistent Reserve Out)  */
/*----------------------------------------------------------------------------*/

#define STIOC_PERS_RESERVE_OUT         _IOWR('T',13, pers_reserve_out_t)

typedef struct {
   uchar reserve_key[8];               /* reservation key                     */
   uchar service_key[8];               /* service action reservation key      */
   uchar scope_addr[4];                /* scope specific address              */
   uchar            : 7,               /* reserved                            */
         aptpl      : 1;               /* activat persist thru power loss     */
   uchar reserved1;                    /* reserved                            */
   uchar obselete[2];                  /* obselete                            */
} pers_out_parm_t;


typedef struct {
   uchar service_action;               /* service action                      */
   uchar scope      : 4,               /* scope                               */
         type       : 4;               /* type                                */
   #ifdef __LP64__
      ptr32_t out_parm;                /* persist reserve out parameter list  */
   #else
      pers_out_parm_t *out_parm;       /* persist reserve out parameter list  */
   #endif /* __LP64__ */
} pers_reserve_out_t;

/*----------------------------------------------------------------------------*/
/* Defines and data structures for SET/GET DENSITY                            */
/*----------------------------------------------------------------------------*/

typedef struct
{
   char density_code;     /*mode sense header density code */
   char default_density;  /*default write density          */
   char pending_density;  /*pending write density          */
   char reserved[9];
} density_data_t;

#define STIOC_SET_DENSITY         _IOW('T',14, density_data_t)
#define STIOC_GET_DENSITY         _IOR('T',15, density_data_t)


/*----------------------------------------------------------------------------*/
/* Defines and data structures for SET/GET ENCRYPTION STATE and SET DATA KEY  */
/*----------------------------------------------------------------------------*/

typedef struct encryption_status {
  uchar encryption_capable; /* Set as a boolean based on the capability of the drive */
                            /* used for GET ioctl only */
  uchar encryption_method;  /* this field is used for GET ioctl only */
     #define METHOD_NONE             0
     #define METHOD_LIBRARY          1
     #define METHOD_SYSTEM           2
     #define METHOD_APPLICATION      3
     #define METHOD_CUSTOM           4
     #define METHOD_UNKNOWN          5

  uchar encryption_state;   /* this field is used for GET and SET ioctl */
     #define STATE_OFF       0
     #define STATE_ON        1
     #define STATE_NA        2
  uchar reserved[13];
} encryption_status_t;

#define GET_ENCRYPTION_STATE      _IOR('T',16, encryption_status_t)
#define SET_ENCRYPTION_STATE      _IOW('T',17, encryption_status_t)

typedef struct data_key {
  uchar data_key_index[12];    /* The DKi */
  uchar data_key_index_length; /* The DKi length */
  uchar reserved1[15];
  uchar data_key[32];          /* The DK  */
  uchar reserved2[48];
} data_key_t;

#define SET_DATA_KEY              _IOW('T',18, data_key_t)


/*-------------------------------------------------------------------------------*/
/* Defines for tape partition ioctls                                             */
/*-------------------------------------------------------------------------------*/

/* Defines for "type" */
#define UNKNOWN_PARTITION   	0      /* Partition type is unknown              */
#define IDP_PARTITION      		1      /* Initiator Defined Partition type       */
#define SDP_PARTITION       	2      /* Select Data Partition type             */
#define FDP_PARTITION       	3      /* Fixed Data Partition type              */

/* Defines for "partition_method" */
#define UNKNOWN_TYPE			0	   /* Vendor-specific or unknown			 */
#define WRAP_WISE_PARTITION 	1	   /* Wrap-wise Partitioning				 */
#define LONGITUDINAL_PARTITION	2	   /* Longitudinal Partitioning				 */

/* Defines for "size_unit" */
#define SIZE_UNIT_BYTES     	0      /* Bytes                                  */
#define SIZE_UNIT_KBYTES    	3      /* Kilobytes                              */
#define SIZE_UNIT_MBYTES    	6      /* Megabytes                              */
#define SIZE_UNIT_GBYTES    	9      /* Gigabytes                              */
#define SIZE_UNIT_TBYTES   		12     /* Terabytes                              */

#define MAX_PARTITIONS     		255    /* Max number of partitions               */

/*-------------------------------------------------------------------------------*/
/* Defines and data structures for query paritition		                         */
/*-------------------------------------------------------------------------------*/



typedef struct query_partition
{
  uchar  max_partitions;           /* Max number of supported partitions */
  uchar  active_partition;         /* current active partition on tape   */
  uchar  number_of_partitions;     /* Number of partitions from 1 to max */
  uchar  size_unit;                /* Size unit of partition sizes below */
  ushort size[MAX_PARTITIONS];     /* Array of partition sizes in size units  */
                                   /* for each partition, 0 to (number - 1)   */
  uchar	 partition_method;		   /* partition type for 3592 E07 and    */
								   /* later generations only	  		 */
  char   reserved[31];
} query_partition_t;

#define QUERY_PARTITION            _IOR('T',34,query_partition_t)

/*---------------------------------------------------------------------------*/
/* Defines and data structures for create paritition                         */
/*---------------------------------------------------------------------------*/



typedef struct tape_partition
{
  uchar  type;                     /* Type of tape partition to create       */
  uchar  number_of_partitions;     /* Number of partitions to create         */
  uchar  size_unit;                /* IDP size unit of partition sizes below */
  ushort size[MAX_PARTITIONS];     /* Array of partition sizes in size units */
                                   /* for each partition, 0 to (number - 1)  */
                                   /* Size can not be 0 and one partition    */
                                   /* size must be 0xFFFF to use the         */
                                   /* remaining capacity on the tape.        */
  uchar	 partition_method;		   /* partitioning type for the 3592 E07 and */
								   /* later generations only                 */
  char   reserved [31];
} tape_partition_t;

#define CREATE_PARTITION           _IOW('T',35,tape_partition_t)

/*---------------------------------------------------------------------------*/
/* Defines and data structures for set active paritition                     */
/*---------------------------------------------------------------------------*/


typedef struct set_active_partition
{
  uchar  partition_number;         /* Partition number 0-n to change to     */
  ullong logical_block_id;         /* Blockid to locate to within partition */
  char   reserved[32];
} set_active_partition_t;

#define SET_ACTIVE_PARTITION      _IOW('T',36,set_active_partition_t)

/*---------------------------------------------------------------------------*/
/* Defines and data structures for allow data overwrite                      */
/*---------------------------------------------------------------------------*/



typedef struct allow_data_overwrite
{
	uchar  partition_number;         /* Partition number 0-n to overwrite     */
	ullong logical_block_id;         /* Blockid to overwrite within partition */
	uchar  allow_format_overwrite;   /* allow format if in data safe mode     */
	char   reserved[32];
} allow_data_overwrite_t;

#define ALLOW_DATA_OVERWRITE      _IOW('T',37,struct allow_data_overwrite)

/*---------------------------------------------------------------------------*/
/* Defines and data structures for partition aware read tape position        */
/*---------------------------------------------------------------------------*/


#define RP_SHORT_FORM         0x00
#define RP_LONG_FORM          0x06
#define RP_EXTENDED_FORM      0x08

typedef struct short_data_format
{
  uchar bop        : 1,              /* beginning of partition                */
        eop        : 1,              /* end of partition                      */
		locu       : 1,              /* 1 means num_buffer_logical_obj field  */
									 /* is unknown                            */
		bycu       : 1,              /* 1 means the num_buffer_bytes field    */
                                     /* is unknown                            */
        rsvd       : 1,
	    lolu       : 1,              /* 1 means the first and last logical    */
									 /* obj position fields are unknown       */
		perr       : 1,              /* 1 means the position fields have      */
                                     /* overflowed and can not be reported    */
        bpew       : 1;              /* beyond programmable early warning     */

  uchar active_partition;            /* current active partition              */
  char  reserved[2];
  uint  first_logical_obj_position;  /* current logical object position       */
  uint  last_logical_obj_position;   /* next logical object to be transferred */
                                     /* to tape */
  uint  num_buffer_logical_obj;      /* number of logical objects in buffer   */
  uint  num_buffer_bytes;            /* number of bytes in buffer             */
  char  reserved1;
} short_data_format_t;


typedef struct long_data_format
{
  uchar  bop        : 1,              /* beginning of partition               */
         eop        : 1,              /* end of partition                     */
         rsvd1      : 2,
		 mpu        : 1,              /* 1 means the logical file id field is */
                                      /* unknown                              */
    	 lonu       : 1,              /* 1 means the partition number or      */
					                  /* logical obj number field is unknown  */
         rsvd2      : 1,
         bpew       : 1;              /* beyond programmable early warning    */

  char   reserved[6];
  uchar  active_partition;            /* current active partition             */
  ullong logical_obj_number;          /* current logical object position      */
  ullong logical_file_id;             /* number of filemarks from bop and     */
                                      /* current logical position             */
  ullong obsolete;
} long_data_format_t;

typedef struct extended_data_format
{
  uchar    bop      : 1,              /* beginning of partition                */
           eop      : 1,              /* end of partition                      */
	       locu     : 1,              /* 1 means num_buffer_logical_obj field  */
				                      /* is unknown */
	       bycu     : 1,              /* 1 means the num_buffer_bytes field is */
				                      /* unknown   */
           rsvd     : 1,
	       lolu     : 1,              /* 1 means the first and last logical    */
                                      /* obj position fields are unknown       */
   	       perr     : 1,              /* 1 means the position fields have      */
                                      /* overflowed and can not be reported    */
           bpew     : 1;              /* beyond programmable early warning     */

  uchar    active_partition;          /* current active partition              */
  ushort   additional_length;
  uint     num_buffer_logical_obj;    /* number of logical objects in buffer   */
  ullong   first_logical_obj_position;/* current logical object position       */
  ullong   last_logical_obj_position; /* next logical object to be transferred */
                                      /* to tape */
  ullong   num_buffer_bytes;          /* number of bytes in buffer             */
  char     reserved;
} extended_data_format_t;

typedef struct read_tape_position
{
	uchar data_format;               /* IN:  Specifies the return data format */
	union                            /* OUT: position data                    */
	{
		short_data_format_t    rp_short;
		long_data_format_t     rp_long;
		extended_data_format_t rp_extended;
		char reserved[64];
	} rp_data;
} read_tape_position_t ;

#define READ_TAPE_POSITION        _IOWR('T',38,read_tape_position_t)

/*---------------------------------------------------------------------------*/
/* Defines and data structures for read tape position                        */
/*---------------------------------------------------------------------------*/


#define LOGICAL_ID_BLOCK_TYPE   0x00
#define LOGICAL_ID_FILE_TYPE    0x01

typedef struct set_tape_position
{
	uchar  logical_id_type;          /* Block or file as defined above       */
	ullong logical_id;               /* logical object or logical file to    */
									 /* position to                          */
	char   reserved[32];
} set_tape_position_t;

#define SET_TAPE_POSITION         _IOW('T',39,set_tape_position_t)


/*---------------------------------------------------------------------------*/
/* Defines and data structures for read logical block protection             */
/*---------------------------------------------------------------------------*/

typedef struct logical_block_protection
{
	uchar lbp_capable;      /* [OUTPUT] the capability of lbp for  QUERY ioctl only      */
	uchar lbp_method;       /* lbp method used for QUERY [OUTPUT] and SET [INPUT] ioctls */
	uchar lbp_info_length;  /* lbp info length for QUERY [OUTPUT] and SET [INPUT] ioctls */
	uchar lbp_w;            /* protection info included in write data                    */
	                        /* a boolean for QUERY [OUTPUT] and SET [INPUT] ioctls       */
	uchar lbp_r;            /* protection info included in read data                     */
	                        /* a boolean for QUERY [OUTPUT] and SET [INPUT] ioctls       */
	uchar rbdp;             /* protection info included in recover buffer data           */
	                        /* a boolean for QUERY [OUTPUT] and SET [INPUT] ioctls       */
	uchar reserved[26];
}logical_block_protection_t;

#define SET_LOGICAL_BLOCK_PROTECTION         _IOWR('T',40,logical_block_protection_t)
#define QUERY_LOGICAL_BLOCK_PROTECTION       _IOWR('T',41,logical_block_protection_t)

#define LBP_DISABLE             0x00
#define REED_SOLOMON_CRC        0x01


/*---------------------------------------------------------------------------*/
/* Defines and data structures for read verify tape data                     */
/*---------------------------------------------------------------------------*/

typedef struct verify_data 
{                                                        
 uchar          :    2,  /* reserved                                                      */
         vte    :    1,  /* [IN] verify to end-of-data                                    */
         vlbpm 	:    1,  /* [IN] verify logical block protection information              */
         vbf   	:    1,  /* [IN] verify by filemarks                                      */
         immed 	:    1,  /* [IN] return SCSI status immediately                           */
         bytcmp	:    1,  /*  No use currently                                             */
         fixed 	:    1;  /* [IN] set Fixed bit to verify the length of each logical block */
 
 uchar   reseved[15];
 uint    verify_length;  /* [IN] amount of data to be verified                            */
} verify_data_t;

#define VERIFY_TAPE_DATA	_IOW('T',42,verify_data_t)	

/*---------------------------------------------------------------------------*/

#endif  /* ATDD_ST_H */




